module Rex
  module Server
    # Implementation of a simple one node server setup
    #
    # Setups/Starts
    #
    # - a message processor to parse and forward
    #   messages to the matching engine
    # - a matching engine instance to process any order book related
    #   messages
    # - a websocket server to accept websocket base connections and
    #   process incoming messages
    class SimpleServerSetup
      def initialize(host, port)
        matching_engine_inbox_pipe = EventMachine::Channel.new
        matching_engine_outbox_pipe = EventMachine::Channel.new

        @matching_engine = MatchingEngine.new(
          matching_engine_inbox_pipe,
          matching_engine_outbox_pipe
        )

        message_processor_inflow_pipe = EventMachine::Channel.new
        message_broker = MessageBroker.new(JsonMessageSerializer.new)

        @message_processor = Rex::Server::MessageProcessor.new(
          matching_engine_inbox_pipe,
          matching_engine_outbox_pipe,
          message_broker,
          message_processor_inflow_pipe
        )

        @websocket_server = Rex::Server::WebsocketServer.new(
          message_broker,
          message_processor_inflow_pipe,
          host: host,
          port: port
        )
      end

      def start
        @message_processor.start
        @matching_engine.start
        @websocket_server.start
      end

      def stop
        @matching_engine.stop
        @message_processor.stop
        @websocket_server.stop
      end
    end
  end
end
